@@ -16,20 +16,23 @@ import subprocess
 import tempfile
 from cookielib import Cookie, LWPCookieJar
 
+import lxml.html as HTML
+
 import sip
 
 sip.setapi(u'QVariant', 2)
 
 from PyQt4.Qt import Qt
 from PyQt4.QtTest import QTest
+
 from PyQt4 import QtWebKit
 from PyQt4.QtNetwork import QNetworkRequest, QNetworkAccessManager, QNetworkProxy, \
     QNetworkCookieJar, QNetworkDiskCache, QNetworkProxyFactory, QNetworkCookie
 from PyQt4.QtWebKit import QWebSettings, QWebPage, QWebView, QWebInspector
 from PyQt4.QtCore import QSize, QByteArray, QUrl, QDateTime, \
-    QtCriticalMsg, QtDebugMsg, QtFatalMsg, QtWarningMsg, QPoint
+    QtCriticalMsg, QtDebugMsg, QtFatalMsg, QtWarningMsg, QPoint, QEvent
 
-from PyQt4.QtGui import QApplication, QImage, QPainter
+from PyQt4.QtGui import QApplication, QImage, QPainter, QMouseEvent
 
 
 logging.basicConfig(level=logging.DEBUG,
@@ -38,7 +41,75 @@ logging.basicConfig(level=logging.DEBUG,
 
 logger = logging.getLogger('GRobot')
 
-import lxml.html as HTML
+default_user_agent = "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:20.0) Gecko/20100101 Firefox/20.0"
+
+_pattern_type = type(re.compile("", 0))
+
+
+def can_load_page(func):
+    """Decorator that specifies if user can expect page loading from
+    this action. If expect_loading is set to True, GRobot will wait
+    for page_loaded event.
+
+    """
+
+    @wraps(func)
+    def wrapper(self, *args, **kwargs):
+        expect_loading = False
+        if 'expect_loading' in kwargs:
+            expect_loading = kwargs['expect_loading']
+            del kwargs['expect_loading']
+        if expect_loading:
+            self.loaded = False
+            func(self, *args, **kwargs)
+            return self.wait_for_page_loaded()
+        return func(self, *args, **kwargs)
+
+    return wrapper
+
+
+def have_a_break(func):
+    """Decorator that specifies if user can expect page loading from
+    this action. If expect_loading is set to True, GRobot will wait
+    for page_loaded event.
+
+    """
+
+    @wraps(func)
+    def wrapper(self, *args, **kwargs):
+        if 'sleep' in kwargs:
+            interval = kwargs['sleep']
+            del kwargs['sleep']
+        else:
+            sleep=self.sleep
+
+        result=func(self, *args, **kwargs)
+        gevent.sleep(sleep)
+        return result
+
+    return wrapper
+
+
+def singleton(class_):
+    class class_w(class_):
+        _instance = None
+
+        def __new__(class_, *args, **kwargs):
+            if class_w._instance is None:
+                class_w._instance = super(class_w, class_).__new__(class_,
+                                                                   *args,
+                                                                   **kwargs)
+                class_w._instance._sealed = False
+            return class_w._instance
+
+        def __init__(self, *args, **kwargs):
+            if self._sealed:
+                return
+            super(class_w, self).__init__(*args, **kwargs)
+            self._sealed = True
+
+    class_w.__name__ = class_.__name__
+    return class_w
 
 
 class XPath(object):
@@ -56,13 +127,10 @@ class XPath(object):
         try:
             return map(lambda x: unicode(x) if isinstance(x, basestring) else x,
                        self.__parser_content.xpath(*arg, **kwargs))
-        except:
+        except Exception:
             return None
 
 
-_pattern_type = type(re.compile("", 0))
-
-
 class GRobotNetworkAccessManager(QNetworkAccessManager):
     def __init__(self, parent=None):
         super(GRobotNetworkAccessManager, self).__init__(parent)
@@ -79,7 +147,6 @@ class GRobotWebPage(QtWebKit.QWebPage):
     """
 
     def __init__(self, robot, parent=None):
-
         super(GRobotWebPage, self).__init__(parent)
         self._robot = robot
         self.setNetworkAccessManager(GRobotNetworkAccessManager(self))
@@ -92,8 +159,6 @@ class GRobotWebPage(QtWebKit.QWebPage):
         log_type = "error" if "Error" in message else "info"
         getattr(logger, log_type)("%s(%d): %s" % (source or '<unknown>', line, message))
 
-
-    #TODO:Need more time to test these.
     def chooseFile(self, frame, suggested_file=None):
         if self._robot._upload_file and os.path.isfile(self._robot._upload_file):
             return self._robot._upload_file
@@ -102,6 +167,7 @@ class GRobotWebPage(QtWebKit.QWebPage):
 
     def javaScriptAlert(self, frame, message):
         """Notifies GRobot for alert, then pass."""
+
         self._robot._alert = message
         self._robot.popup_messages = message
         logger.debug("alert('%s')" % message)
@@ -110,6 +176,7 @@ class GRobotWebPage(QtWebKit.QWebPage):
         """Checks if GRobot is waiting for confirm, then returns the right
         value.
         """
+
         if self._robot._confirm_expected is None:
             raise Exception('You must specified a value to confirm "%s"' %
                             message)
@@ -126,6 +193,7 @@ class GRobotWebPage(QtWebKit.QWebPage):
         """Checks if GRobot is waiting for prompt, then enters the right
         value.
         """
+
         if self._robot._prompt_expected is None:
             raise Exception('You must specified a value for prompt "%s"' %
                             message)
@@ -153,71 +221,6 @@ class GRobotWebPage(QtWebKit.QWebPage):
         return self.user_agent
 
 
-default_user_agent = "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:20.0) Gecko/20100101 Firefox/20.0"
-
-
-def can_load_page(func):
-    """Decorator that specifies if user can expect page loading from
-    this action. If expect_loading is set to True, GRobot will wait
-    for page_loaded event.
-
-    """
-
-    @wraps(func)
-    def wrapper(self, *args, **kwargs):
-        expect_loading = False
-        if 'expect_loading' in kwargs:
-            expect_loading = kwargs['expect_loading']
-            del kwargs['expect_loading']
-        if expect_loading:
-            self.loaded = False
-            func(self, *args, **kwargs)
-            return self.wait_for_page_loaded()
-        return func(self, *args, **kwargs)
-
-    return wrapper
-
-
-def need_webview(func):
-    """Decorator that specifies if user can expect page loading from
-    this action. If expect_loading is set to True, GRobot will wait
-    for page_loaded event.
-
-    """
-
-    @wraps(func)
-    def wrapper(self, *args, **kwargs):
-        if not self.display:
-            self.webview = self.webview or QtWebKit.QWebView()
-            self.webview.setPage(self.page)
-            self.webview.hide()
-        return func(self, *args, **kwargs)
-
-    return wrapper
-
-
-def singleton(class_):
-    class class_w(class_):
-        _instance = None
-
-        def __new__(class_, *args, **kwargs):
-            if class_w._instance is None:
-                class_w._instance = super(class_w, class_).__new__(class_,
-                                                                   *args,
-                                                                   **kwargs)
-                class_w._instance._sealed = False
-            return class_w._instance
-
-        def __init__(self, *args, **kwargs):
-            if self._sealed:
-                return
-            super(class_w, self).__init__(*args, **kwargs)
-            self._sealed = True
-
-    class_w.__name__ = class_.__name__
-    return class_w
-
-
 @singleton
 class QtMainLoop(object):
     """The Qt main loop thread.Singleton pattern.
@@ -234,9 +237,9 @@ class QtMainLoop(object):
 
         self._app.exit()
         #TODO:This is very ugry to hard-code terminate time.But I can't find another better way.
-        gevent.sleep(1)
+        # gevent.sleep(1)
         self._stop = True
-        self._greenlet.join(timeout=20)
+        self._greenlet.join(timeout=10)
         self._greenlet = None
         sip.delete(self._app)
 
@@ -255,8 +258,8 @@ class QtMainLoop(object):
                 # GRobot._app.processEvents()
                 while self._app.hasPendingEvents():
                     self._app.processEvents()
-                    gevent.sleep(0)
-                gevent.sleep(1)
+                    # gevent.sleep(0)
+                gevent.sleep(0)
         except Exception, e:
             logger.error(e)
         logger.debug('Goodbye GRobot')
@@ -273,9 +276,9 @@ class HttpResource(object):
         self.content = content
         if self.content is None:
             # Tries to get back content from cache
-            buffer = cache.data(self.url)
-            if buffer is not None:
-                content = buffer.readAll()
+            _buffer = cache.data(self.url)
+            if _buffer is not None:
+                content = _buffer.readAll()
                 try:
                     self.content = unicode(content)
                 except UnicodeDecodeError:
@@ -353,7 +356,7 @@ class GRobot(object):
                  display=False, viewport_size=(1024, 768), accept_language='en,*', ignore_ssl_errors=True,
                  cache_dir=os.path.join(tempfile.gettempdir(), "GRobot"),
                  image_enabled=True, plugins_enabled=False, java_enabled=False, javascript_enabled=True,
-                 plugin_path=None, develop=False, proxy=None):
+                 plugin_path=None, develop=False, proxy=None,sleep=0.5):
         """GRobot manages a QWebPage.
     
         @param user_agent: The default User-Agent header.
@@ -372,6 +375,7 @@ class GRobot(object):
         @param plugin_path: Array with paths to plugin directories (default ['/usr/lib/mozilla/plugins'])
         @param develop: Enable the Webkit Inspector.
         @param proxy: Set a Socks5,HTTP{S} Proxy
+        @param sleep: Sleep `sleep` second,after operate
         """
 
         GRobot.exit_lock.acquire()
@@ -383,6 +387,7 @@ class GRobot(object):
         self.plugin = False
         self.exitLoop = False
         self.set_proxy(proxy)
+        self.sleep=sleep
 
         self.popup_messages = None
 
@@ -466,10 +471,12 @@ class GRobot(object):
 
         self.webview = None
 
-        if self.display:
-            self.webview = QtWebKit.QWebView()
-            self.webview.setPage(self.page)
-            self.webview.show()
+        self.viewport_size = viewport_size
+
+        self.webview = QtWebKit.QWebView()
+        self.webview.setPage(self.page)
+
+        self.webview.show() if display else self.webview.hide()
 
         self.set_viewport_size(*viewport_size)
 
@@ -618,122 +625,272 @@ class GRobot(object):
         else:
             QNetworkProxyFactory.setUseSystemConfiguration(True)
 
+    def first_element_position(self, selector):
+        try:
+            return self.elements_position(selector)[0]
+        except IndexError:
+            logger.warning("Can't locate selector " + selector)
+            return None
 
-    def get_postions_from_selector(self, selector):
-        if selector.startswith('xpath='):
-            query = selector[6:]
-            postions = self.evaluate(u"""
+
+    def elements_position(self, selector):
+        attr, pattern, val = self.parser_selector(selector, attr='identifier')
+
+        strip = lambda v: v.strip()
+
+        if pattern:
+            val = locals()[pattern](val)
+
+
+        def identifier(query):
+            return id(query) or name(query)
+
+        def name(query):
+            return css("*[name='%s']" % query)
+
+        def id(query):
+            return css('#' + query)
+
+        def link(query):
+            return xpath(u"//a[@text()='%s']" % query.replace("\'", "\\'"))
+
+        def css(query):
+            result = []
+            for ele in self.main_frame.findAllElements(query):
+                if not ele.isNull():
+                    result.append(ele.geometry().center())
+            return result
+
+        def xpath(query):
+            positions = self.evaluate(u"""
             function GetAbsoluteLocationEx(element)
-{
-    if ( arguments.length != 1 || element == null )
-    {
-        return null;
-    }
-    var elmt = element;
-    var offsetTop = elmt.offsetTop;
-    var offsetLeft = elmt.offsetLeft;
-    var offsetWidth = elmt.offsetWidth;
-    var offsetHeight = elmt.offsetHeight;
-    while( elmt = elmt.offsetParent )
-    {
-          // add this judge
-        if ( elmt.style.position == 'absolute' || elmt.style.position == 'relative'
-            || ( elmt.style.overflow != 'visible' && elmt.style.overflow != '' ) )
-        {
-            break;
-        }
-        offsetTop += elmt.offsetTop;
-        offsetLeft += elmt.offsetLeft;
-    }
-    return { absoluteTop: offsetTop, absoluteLeft: offsetLeft,
-        offsetWidth: offsetWidth, offsetHeight: offsetHeight };
-}
-
-
-                result=[];
-                for (var r = document.evaluate('%s', document, null, 5, null), n; n = r.iterateNext();) {
-                //result.push([n.scrollLeft+n.offsetWidth/2.0,n.scrollTop+n.offsetHeight/2.0]);
-                pos=GetAbsoluteLocationEx(n)
-                pos
-                result.push([pos.absoluteLeft+pos.offsetWidth/2.0,pos.absoluteTop+pos.offsetHeight/2.0]);
+            {
+                if ( arguments.length != 1 || element == null )
+                {
+                    return null;
                 }
-                result
+                var elmt = element;
+                var offsetTop = elmt.offsetTop;
+                var offsetLeft = elmt.offsetLeft;
+                var offsetWidth = elmt.offsetWidth;
+                var offsetHeight = elmt.offsetHeight;
+                while( elmt = elmt.offsetParent )
+                {
+                      // add this judge
+                    if ( elmt.style.position == 'absolute' || elmt.style.position == 'relative'
+                        || ( elmt.style.overflow != 'visible' && elmt.style.overflow != '' ) )
+                    {
+                        break;
+                    }
+                    offsetTop += elmt.offsetTop;
+                    offsetLeft += elmt.offsetLeft;
+                }
+                return { absoluteTop: offsetTop, absoluteLeft: offsetLeft,
+                    offsetWidth: offsetWidth, offsetHeight: offsetHeight };
+            }
+            result=[];
+            for (var r = document.evaluate('%s', document, null, 5, null), n; n = r.iterateNext();) {
+            pos=GetAbsoluteLocationEx(n)
+            result.push([pos.absoluteLeft+pos.offsetWidth/2.0,pos.absoluteTop+pos.offsetHeight/2.0]);
+            }
+            result
             """ % query.replace("\'", "\\'"))
-            return map(lambda x: tuple(x), postions)
-        elif selector.startswith('css='):
-            query = selector[4:]
 
-            def qpoint_to_tuple(point):
-                return (point.x(), point.y())
+            return map(lambda x: QPoint(*tuple(x)), positions)
 
-            return [qpoint_to_tuple(x.geometry().center()) for x in self.main_frame.findAllElements(query)]
-        elif selector.startswith('id='):
-            query = selector[3:]
-            return self.get_postions_from_selector('css=#' + query)
-        elif selector.startswith('link='):
-            query = selector[5:]
-            return self.get_postions_from_selector("xpath=//a[@text()='%s']" + query)
-        elif selector.startswith('dom='):
-            query = selector[4:]
-            raise NotImplementedError
-        else:
-            raise NotImplementedError
+        return locals()[attr](val)
 
 
-    def set_page_center(self, postion):
-        width = self.evaluate("document.documentElement.clientWidth")
-        height = self.evaluate("document.documentElement.clientHeight")
-        self.main_frame.setScrollPosition(QPoint(postion[0] - width / 2.0, postion[1] - height / 2.0))
+    def _move_page_center_to(self, qpoint):
+        size = self.page.viewportSize()
+        self.main_frame.setScrollPosition(qpoint - QPoint(size.width(), size.height()) / 2)
 
 
-
-    @need_webview
     def reload(self):
         """Reload page.
 
         @return:
         """
-        self.loaded = False
-        self.page.triggerAction('Reload')
-        return self.wait_for_page_loaded()
 
-    @need_webview
+        self.trigger_action('Reload',expect_loading=True)
+
+    def back(self):
+        self.trigger_action('Back')
+
+    def forward(self):
+        self.trigger_action('Forward')
+
+
+    @can_load_page
     def trigger_action(self, action):
+        """Trigger QWebPage::WebAction
+
+        @param action:
+        """
         self.page.triggerAction(getattr(QWebPage, action))
 
 
-    @need_webview
-    @can_load_page
-    def click(self, selector):
-        postions = self.get_postions_from_selector(selector)
-        if postions:
-            postion=postions[0]
-            self.set_page_center(postion)
-            pos = QPoint(
-                postion[0]-self.main_frame.scrollPosition().x(),
-                postion[1]-self.main_frame.scrollPosition().y(),
-                         )
-            QTest.mouseMove(self.webview, pos=pos, delay=100)
-            QTest.mouseClick(self.webview, Qt.LeftButton, pos=pos, delay=100)
+    def parser_selector(self, selector, attr=None, pattern=None, val=None):
+        index = selector.find('=')
+
+        if index <= 0:
+            val = selector
         else:
-            logger.warning("Can't locate selector " + selector)
+            attr = selector[:index]
+            value_ = selector[index + 1:]
+            index = value_.find(':')
+
+            if index > 0:
+                pattern = value_[:index]
+
+            val = value_[index + 1:]
+
+        return attr, pattern, val
 
 
-    def type(self, text, selector=None, delay=20):
+    @can_load_page
+    @have_a_break
+    def click(self, selector):
+        qpoint = self.first_element_position(selector)
+        if qpoint:
+            return self._click_position(qpoint)
+
+
+    @can_load_page
+    def _click_position(self, qpoint):
+        self._move_page_center_to(qpoint)
+        self.webview.repaint()
+        pos = qpoint - self.main_frame.scrollPosition()
+
+        self._move_to_position(pos)
+        QTest.mouseClick(self.webview, Qt.LeftButton, pos=pos)
+        gevent.sleep(1)
+        return pos
+
+
+    def qpoint_to_tuple(self,qpoint):
+        return qpoint.x(),qpoint.y()
+
+    @have_a_break
+    def move_to(self, selector):
+        qpoint = self.first_element_position(selector)
+        if qpoint:
+            self._move_to_position(qpoint)
+            return qpoint_to_tuple(qpoint)
+
+    def move_at(self, x, y):
+        self._move_to_position(QPoint(x, y))
+
+    def _move_to_position(self, qpoint):
+        QTest.mouseMove(self.webview, pos=qpoint)
+        return qpoint
+
+    @have_a_break
+    def click_at(self, x, y):
+        self._click_position(QPoint(x, y))
+
+    @have_a_break
+    def key_clicks(self, selector, text):
         if selector:
             self.click(selector)
-        QTest.keyClicks(self.webview, text, delay=delay)
+        QTest.keyClicks(self.webview, text,delay=50)
+
+    @have_a_break
+    def type(self, selector, text):
+        position = self.click(selector)
+
+        ele = self._hit_element_from(position)
+
+        ele.setFocus()
+        ele.evaluateJavaScript(
+            u"""
+            core.events.setValue(this, '%s')
+            """ % (text.replace("\n", "\\n").replace("\'", "\\'"))
+        )
+        logger.debug('type %s %s'%(selector,text))
 
 
+    def _hit_element_from(self, position):
+        return self.main_frame.hitTestContent(position).element()
+
+    def first_element(self, selector):
+        position = self.first_element_position(selector)
+        if position:
+            return self.main_frame.hitTestContent(position).element(), position
 
 
-    def set_file_input(self, selector, file):
+    def wait_forever(self):
+        self.wait_for(lambda :False,time_for_stop=-1)
 
+    @have_a_break
+    def check(self, selector, checked=True):
+        ele, position = self.first_element(selector)
+        if ele and ele.tagName() == 'INPUT':
+            if ele.attribute('type') in ['checkbox', 'radio']:
+                ele_checked = ele.attribute('checked') == 'checked' or False
+                if ele_checked != checked:
+                    self._click_position(position)
+            else:
+                raise ValueError, "%s is not a checkbox or radio" % selector
+
+    @have_a_break
+    def select(self, selector, value):
+
+        def _select(query, select_by, select):
+            select.evaluateJavaScript(u"""
+            triggerEvent(this, 'focus', false);
+            var changed = false;
+            var optionToSelect = '%s';
+            for (var i = 0; i < this.options.length; i++) {
+                var option = this.options[i];
+                if (option.selected && option.%s != optionToSelect) {
+                    option.selected = false;
+                    changed = true;
+                }
+                else if (!option.selected && option.%s == optionToSelect) {
+                    option.selected = true;
+                    changed = true;
+                }
+            }
+
+            if (changed) {
+                triggerEvent(this, 'change', true);
+            }
+            """ % ( query.replace("\'", "\\'"), select_by, select_by))
+
+        def _add_selection(query, select_by, select,selected):
+            select.evaluateJavaScript(u"""
+            triggerEvent(this, 'focus', false);
+            var optionToSelect = '%s';
+            for (var i = 0; i < this.options.length; i++) {
+                var option = this.options[i];
+                if (option.%s == optionToSelect)
+                {
+                    option.selected = %s;
+                    triggerEvent(this, 'change', true);
+                }
+
+            }
+            """ % ( query.replace("\'", "\\'"), select_by, selected and 'true' or 'false'))
+
+        ele, position = self.first_element(selector)
+
+        if ele and ele.tagName() == 'SELECT':
+            ele.setFocus()
+
+            if ele.attribute('multiple') == 'multiple':
+                assert isinstance(value,list)
+                for value_,selected in value:
+                    attr, pattern, val = self.parser_selector(value_, attr='text')
+                    _add_selection(val, attr, ele,selected)
+            else:
+                attr, pattern, val = self.parser_selector(value, attr='text')
+                _select(val, attr, ele)
+
+
+    def choose_file(self, selector, file):
         self._upload_file = file
-
-        # for ele in self.main_frame.findAllElements(selector):
         self.click(selector)
-
         self._upload_file = None
 
 
@@ -777,7 +934,6 @@ class GRobot(object):
 
         return result
 
-
     def capture_to_buf(self, selector=None):
         """capture the images of selector to StringIO
 
@@ -798,60 +954,6 @@ class GRobot(object):
 
         return result
 
-    @can_load_page
-    def seleniumChain(self, commands, interval=1):
-        """
-
-        @param commands:
-        @param interval:
-        @return: Lastest page and resources.
-        """
-        for command in commands:
-            self.selenium(*command, interval=interval)
-
-    @can_load_page
-    def selenium(self, command, target='', value='', interval=1):
-        """Using the prowerful selenium.
-        You can find more command reference via http://release.seleniumhq.org/selenium-core/1.0.1/reference.html
-
-
-        @param command: The selenium command.
-        @param target:
-        @param value:
-        @param interval:Sleep `interval` seconds after the command.
-        @return:
-        """
-
-        ss = u"do_selenium_command('%s','%s','%s')" % (
-            command.replace("\'", "\\'"), target.replace("\'", "\\'"), value.replace("\'", "\\'").replace("\n", "\\n"),
-        )
-        # ss = u"""var command=new SeleniumCommand('%s','%s','%s', false);
-        #     var handler=GCrawlerFactory.getCommandHandler(command.command);
-        #     if (handler)
-        #     {
-        #         command.target = selenium.preprocessParameter(command.target);
-        #         command.value = selenium.preprocessParameter(command.value);
-        #
-        #         try
-        #         {
-        #             handler.execute(selenium, command);
-        #         }catch(e)
-        #         {
-        #             if (e.isSeleniumError)
-        #                 e.message;
-        #         }
-        #
-        #     }else
-        #     {
-        #         'Do not hava command '+command.command;
-        #     }
-        #     """ % (
-        #     command.replace("\'", "\\'"), target.replace("\'", "\\'"), value.replace("\'", "\\'").replace("\n", "\\n"),
-        # )
-        logger.debug("Command:%s\tTarget:%s\tValue:%s" % (command, target, value))
-        result = self.evaluate(ss)
-        gevent.sleep(interval)
-        return result
 
     @can_load_page
     def evaluate(self, script):
@@ -874,8 +976,7 @@ class GRobot(object):
         self.evaluate(codecs.open(path, encoding=encoding).read())
 
     def __del__(self):
-        """
-        Depend on the CG of Python.
+        """Depend on the CG of Python.
         """
         self._exit()
 
@@ -1074,19 +1175,6 @@ class GRobot(object):
         return self.wait_for(lambda: self.loaded,
                              'Unable to load requested page', time_for_stop=time_for_stop)
 
-        # resources = self._release_last_resources()
-
-        # page = None
-        # url = str(self.main_frame.url().toString().toUtf8())
-        # url_without_hash = url.split("#")[0]
-
-        # print self.http_resources
-        #
-        # page=self.filter_resources(url) or self.filter_resources(url_without_hash)
-        #
-        #
-        # return page[0]
-
     def wait_for(self, condition, timeout_message='', time_for_stop=None):
         """Waits until condition is True.
 
@@ -1106,7 +1194,7 @@ class GRobot(object):
                     raise OperateTimeout, timeout_message
                 else:
                     # raise LoadingTimeout, timeout_message
-                    self.page.triggerAction(10) #QWebPage::Stop
+                    self.trigger_action('Stop') #QWebPage::Stop
                     self.loaded = True
                     logger.warning("Page loading timeout.Force to stop the page")
                     break
@@ -1138,57 +1226,56 @@ class GRobot(object):
             self.inspector.setPage(self.page)
             self.inspector.show()
 
-        selenium_scripts = ['atoms.js',
-                            'htmlutils.js',
-                            'selenium-logging.js',
-
-                            'find_matching_child.js',
-                            'selenium-api.js',
-                            'selenium-api-override.js',
-
-                            'selenium-browserbot.js',
-                            'selenium-browserdetect.js',
-                            'selenium-commandhandlers.js',
-                            'xmlextras.js', ]
+        selenium_scripts = [
+            'atoms.js',
+            'htmlutils.js',
+            # 'selenium-logging.js',
+            #
+            # 'find_matching_child.js',
+            # 'selenium-api.js',
+            # 'selenium-api-override.js',
+            #
+            # 'selenium-browserbot.js',
+            # 'selenium-browserdetect.js',
+            # 'selenium-commandhandlers.js',
+            # 'xmlextras.js',
+        ]
 
         for script in selenium_scripts:
-            # js_file = open(os.path.dirname(__file__) + '/../selenium-scripts/' + script).read()
-            # self.main_frame.evaluateJavaScript(js_file)
             self.evaluate_js_file(os.path.dirname(__file__) + '/../selenium-scripts/' + script)
 
-        self.evaluate(
-            """
-
-            selenium=Selenium.createForWindow(window);
-            GCrawlerFactory = new CommandHandlerFactory();
-            GCrawlerFactory.registerAll(selenium);
-
-            function do_selenium_command(command,target,value)
-            {
-
-                var command=new SeleniumCommand(command,target,value, false);
-                var handler=GCrawlerFactory.getCommandHandler(command.command);
-                if (handler)
-                {
-                    command.target = selenium.preprocessParameter(command.target);
-                    command.value = selenium.preprocessParameter(command.value);
-
-                try
-                {
-                    handler.execute(selenium, command);
-                }catch(e)
-                {
-                    if (e.isSeleniumError)
-                        e.message;
-                }
-
-                }else
-                {
-                    'Do not hava command '+command.command;
-                }
-            }
-
-""")
+#         self.evaluate(
+#             """
+#             selenium=Selenium.createForWindow(window);
+#             GCrawlerFactory = new CommandHandlerFactory();
+#             GCrawlerFactory.registerAll(selenium);
+#
+#             function do_selenium_command(command,target,value)
+#             {
+#
+#                 var command=new SeleniumCommand(command,target,value, false);
+#                 var handler=GCrawlerFactory.getCommandHandler(command.command);
+#                 if (handler)
+#                 {
+#                     command.target = selenium.preprocessParameter(command.target);
+#                     command.value = selenium.preprocessParameter(command.value);
+#
+#                 try
+#                 {
+#                     handler.execute(selenium, command);
+#                 }catch(e)
+#                 {
+#                     if (e.isSeleniumError)
+#                         e.message;
+#                 }
+#
+#                 }else
+#                 {
+#                     'Do not hava command '+command.command;
+#                 }
+#             }
+#
+# """)
 
         self.loaded = True
         # self.cache.clear()
@@ -1211,13 +1298,9 @@ class GRobot(object):
 
     def _link_clicked(self, href):
         """Contorl the page link clicked event,forbid open new window.
+
         @param href: The href attribute of a tag.
         """
-        # href_string = href.toString()
-
-        #is a real link?It must be a real link.
-        # if not href_string.startsWith('#') and not href_string.startsWith('jacascript:'):
-        #     self.main_frame.load(QUrl(href_string))
 
         self.main_frame.load(href)
 
@@ -1226,6 +1309,7 @@ class GRobot(object):
 
         @param reply: The QNetworkReply object.
         """
+
         if reply.attribute(QNetworkRequest.HttpStatusCodeAttribute):
             self.http_resources.append(HttpResource(reply, self.cache))
 
@@ -1242,11 +1326,16 @@ class GRobot(object):
             self._auth_attempt += 1
 
     def _on_manager_ssl_errors(self, reply, errors):
+        """Ingore all the ssl error
+
+        @param reply:
+        @param errors:
+        """
         url = unicode(reply.url().toString())
         if self.ignore_ssl_errors:
             reply.ignoreSslErrors()
         else:
-            logger.log('SSL certificate error: %s' % url, level='warning')
+            logger.warning('SSL certificate error: %s' % url)
 
 
     def _exit(self):
@@ -1263,10 +1352,7 @@ class GRobot(object):
             sip.delete(self.webview)
 
         if self.page and not sip.isdeleted(self.page):
-            # self.page.deleteLater()
             sip.delete(self.page)
-            # sip.delete(self.main_frame)
-            # sip.delete(self.cache)
 
         GRobot._liveRobot -= 1
 
